import fs from "fs";
import path from "path";
import commentJSON from "comment-json";
import { plugin as workspacesPlugin } from "./manager/workspaces";
import assert from "assert";
export const toProjectReferences = (options) => {
    const plugins = Array.isArray(options.plugins) && options.plugins.length > 0 ? options.plugins : [workspacesPlugin];
    const pluginImplementations = plugins.map((plugin) => plugin(options));
    // use first plugin
    const supportPlugin = pluginImplementations.find((plugin) => {
        return plugin.supports();
    });
    if (!supportPlugin) {
        throw new Error("Not found supported plugin");
    }
    const relativeName = (pkgPath) => {
        return path.relative(options.rootDir, pkgPath);
    };
    const allPackages = supportPlugin.getAllPackages();
    const errors = [];
    allPackages.forEach((packageInfo) => {
        var _a, _b, _c;
        const tsconfigFilePath = (_b = (_a = options.tsConfigPathFinder) === null || _a === void 0 ? void 0 : _a.call(options, packageInfo.location)) !== null && _b !== void 0 ? _b : path.join(packageInfo.location, "tsconfig.json");
        if (!fs.existsSync(tsconfigFilePath)) {
            // Skip has not tsconfig.json
            return;
        }
        const tsconfigJSON = commentJSON.parse(fs.readFileSync(tsconfigFilePath, "utf-8"));
        const references = supportPlugin.getDependencies(packageInfo.packageJSON);
        const newProjectReferences = references
            .map((reference) => {
            const absolutePathOrNull = supportPlugin.resolve(reference);
            if (!absolutePathOrNull) {
                return;
            }
            if (!path.isAbsolute(absolutePathOrNull)) {
                throw new Error(`Plugin#resolve should return absolute path: ${absolutePathOrNull}, plugin: ${supportPlugin}`);
            }
            if (packageInfo.location === absolutePathOrNull) {
                const selfName = relativeName(packageInfo.location);
                errors.push(new Error(`[${selfName}] Self dependencies is something wrong: ${selfName} refer to ${relativeName(absolutePathOrNull)}`));
            }
            return {
                path: path.relative(packageInfo.location, absolutePathOrNull)
            };
        })
            .filter((r) => Boolean(r));
        const currentProjectReferences = (_c = tsconfigJSON["references"]) !== null && _c !== void 0 ? _c : [];
        if (options.checkOnly) {
            // check
            try {
                // create pure object for compare
                const cleanCurrentProjectReferences = JSON.parse(JSON.stringify(commentJSON.parse(commentJSON.stringify(currentProjectReferences), undefined, true)));
                assert.deepStrictEqual(cleanCurrentProjectReferences, newProjectReferences);
            }
            catch (error) {
                const selfName = relativeName(packageInfo.location);
                errors.push(new Error(`[${selfName}] ${error.message}`));
            }
            return;
        }
        else {
            // update
            if (errors.length === 0) {
                tsconfigJSON["references"] = newProjectReferences;
                fs.writeFileSync(tsconfigFilePath, commentJSON.stringify(tsconfigJSON, null, 2), "utf-8");
            }
        }
    });
    if (errors.length > 0) {
        return {
            ok: false,
            aggregateError: {
                message: `Found ${errors.length} errors`,
                errors
            }
        };
    }
    return {
        ok: true
    };
};
//# sourceMappingURL=index.js.map